param (
    [Parameter(Mandatory=$false, HelpMessage="Parameter help description")]
    [string]$CmsUrl = "http://localhost:81/",

    [Parameter(Mandatory=$false, HelpMessage="Authentication type")]
    [ValidateSet("Windows","Basic")]
    [string]$AuthType = "Windows",

    [Parameter(Mandatory=$false)]
    [string]$userName = $null,

    [Parameter(Mandatory=$false)]
    [string]$userPass = $null
)

$masterCoreModuleLocationUrl = '/webdav/100%20Master/Building%20Blocks/Modules/Core'
$siteTypeCoreModuleLocationUrl = '/webdav/110%20DXA%20Site%20Type/Building%20Blocks/Modules/Core'
$siteTypeHomeSgLocationUrl = '/webdav/110%20DXA%20Site%20Type/Home'
$exampleSiteLocationUrl = '/webdav/400%20Example%20Site'

$ItemMapperStorage = @{
"PageToSpecificPageTemplateMapper" = @{
    "$siteTypeHomeSgLocationUrl/_Page%20Types/Article%20Page.tpg" = "$masterCoreModuleLocationUrl/Editor/Templates/News%20Page.tptcmp"
    "$siteTypeHomeSgLocationUrl/_Page%20Types/News%20Article%20Page.tpg" = "$masterCoreModuleLocationUrl/Editor/Templates/News%20Page.tptcmp"
    "$exampleSiteLocationUrl/Home/010%20Articles/010%20News/Example%20News%202.tpg" = "$masterCoreModuleLocationUrl/Editor/Templates/News%20Page.tptcmp"
    "$exampleSiteLocationUrl/Home/010%20Articles/010%20News/Example%20News%203.tpg" = "$masterCoreModuleLocationUrl/Editor/Templates/News%20Page.tptcmp"
    "$exampleSiteLocationUrl/Home/010%20Articles/010%20News/Example%20News%201.tpg" = "$masterCoreModuleLocationUrl/Editor/Templates/News%20Page.tptcmp"
    "$exampleSiteLocationUrl/Home/010%20Articles/010%20News/Example%20News%204.tpg" = "$masterCoreModuleLocationUrl/Editor/Templates/News%20Page.tptcmp"
    "$exampleSiteLocationUrl/Home/_System/include/Footer.tpg" = "$exampleSiteLocationUrl/Building%20Blocks/Modules/Core/Editor/Templates/Include%20Page%20%28Footer%29.tptcmp"
    "$exampleSiteLocationUrl/Home/_System/include/Header.tpg" = "$exampleSiteLocationUrl/Building%20Blocks/Modules/Core/Editor/Templates/Include%20Page%20%28Header%29.tptcmp"
    "$siteTypeHomeSgLocationUrl/_System/include/Footer.tpg" = "$siteTypeCoreModuleLocationUrl/Editor/Templates/Include%20Page%20%28Footer%29.tptcmp"
    "$siteTypeHomeSgLocationUrl/_System/include/Header.tpg" = "$siteTypeCoreModuleLocationUrl/Editor/Templates/Include%20Page%20%28Header%29.tptcmp"
    "$siteTypeHomeSgLocationUrl/_System/include/Left%20Navigation.tpg" = "$siteTypeCoreModuleLocationUrl/Editor/Templates/Include%20Page%20%28Left%29.tptcmp"
    "$siteTypeHomeSgLocationUrl/_System/include/Content%20Tools.tpg" = "$siteTypeCoreModuleLocationUrl/Editor/Templates/Include%20Page%20%28Tools%29.tptcmp"
}

"PageTemplateMapper" = @{
    "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Include%20Page.tptcmp" = "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Include%20Page.tptcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/Content%20Page.tptcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/Content%20Page.tptcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/Content%20Page%20Without%20Navigation.tptcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/Content%20Page%20Without%20Navigation.tptcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/Home%20Page.tptcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/Home%20Page.tptcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/Redirect%20Page.tptcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/Redirect%20Page.tptcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/Section%20Page.tptcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/Section%20Page.tptcmp"
}

"ComponentTemplateMapper" = @{
    "$masterCoreModuleLocationUrl/Editor/Templates/YouTube%20Video%20%5B3-Column%5D.tctcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/YouTube%20Video.tctcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/YouTube%20Video%20%5B2-Column%5D.tctcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/YouTube%20Video.tctcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/Teaser%20Map%20%5B3-Column%5D.tctcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/Teaser%20Map.tctcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/Teaser%20Map%20%5B2-Column%5D.tctcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/Teaser%20Map.tctcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/Teaser%20Hero%20-%20Image%20Overlay%20%5BHero%5D.tctcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/Teaser%20-%20Image%20Overlay.tctcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/Teaser%20%5B3-Column%5D.tctcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/Teaser.tctcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/Teaser%20%5B2-Column%5D.tctcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/Teaser.tctcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/Teaser%20-%20Image%20Overlay%20%5B3-Column%5D.tctcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/Teaser%20-%20Image%20Overlay.tctcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/Teaser%20-%20Image%20Overlay%20%5B2-Column%5D.tctcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/Teaser%20-%20Image%20Overlay.tctcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/List%20%5B3-Column%5D.tctcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/List.tctcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/List%20%5B2-Column%5D.tctcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/List.tctcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/Carousel%20%5BHero%5D.tctcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/Carousel.tctcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/Accordion%20%5B3-Column%5D.tctcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/Accordion.tctcmp"
    "$masterCoreModuleLocationUrl/Editor/Templates/Accordion%20%5B2-Column%5D.tctcmp" = "$masterCoreModuleLocationUrl/Editor/Templates/Accordion.tctcmp"
    "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Top%20Navigation%20%5BNav%5D.tctcmp" = "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Top%20Navigation.tctcmp"
    "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Social%20Sharing%20%5BTools%5D.tctcmp" = "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Social%20Sharing.tctcmp"
    "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Social%20Links%20%5B4-Column%5D.tctcmp" = "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Social%20Links.tctcmp"
    "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Social%20Links%20%5B3-Column%5D.tctcmp" = "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Social%20Links.tctcmp"
    "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Old%20Browser%20Notification%20Bar%20%5BInfo%5D.tctcmp" = "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Old%20Browser%20Notification%20Bar.tctcmp"
    "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Left%20Navigation%20%5BLeft%5D.tctcmp" = "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Left%20Navigation.tctcmp"
    "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Language%20Selector%20%5BNav%5D.tctcmp" = "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Language%20Selector.tctcmp"
    "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Header%20Logo%20%5BLogo%5D.tctcmp" = "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Header%20Logo.tctcmp"
    "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Header%20Links%20%5BNav%5D.tctcmp" = "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Header%20Links.tctcmp"
    "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Footer%20Links%20%5BLinks%5D.tctcmp" = "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Footer%20Links.tctcmp"
    "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Footer%20Link%20Group%20%5B4-Column%5D.tctcmp" = "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Footer%20Link%20Group.tctcmp"
    "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Cookie%20Notification%20Bar%20%5BInfo%5D.tctcmp" = "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Cookie%20Notification%20Bar.tctcmp"
    "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Breadcrumb%20%5BTools%5D.tctcmp" = "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Breadcrumb.tctcmp"
}

"ComponentPresentationDestinationRegionMap" = @{
    "$masterCoreModuleLocationUrl/Editor/Templates/Content%20Page.tptcmp#" =  @{
        RegionName = "Main" }
    "$masterCoreModuleLocationUrl/Editor/Templates/Content%20Page.tptcmp#Main" =  @{
        RegionName = "Main" }
    "$masterCoreModuleLocationUrl/Editor/Templates/Content%20Page%20Without%20Navigation.tptcmp#" = @{
        RegionName = "Main" }
    "$masterCoreModuleLocationUrl/Editor/Templates/Content%20Page%20Without%20Navigation.tptcmp#Main" = @{
        RegionName = "Main" }
    "$masterCoreModuleLocationUrl/Editor/Templates/Home%20Page.tptcmp#Hero" = @{
        RegionName = "Hero" }
    "$masterCoreModuleLocationUrl/Editor/Templates/Home%20Page.tptcmp#" = @{
        RegionName = "Article" }
    "$masterCoreModuleLocationUrl/Editor/Templates/Home%20Page.tptcmp#3-Column" = @{
        RegionName = "Further Info"
        Metadata = [ordered]@{
            numberOfColumns = 3}
    }
    "$exampleSiteLocationUrl/Building%20Blocks/Modules/Core/Editor/Templates/Include%20Page%20%28Footer%29.tptcmp#Link" = @{
        RegionName = "Links" }
    "$exampleSiteLocationUrl/Building%20Blocks/Modules/Core/Editor/Templates/Include%20Page%20%28Footer%29.tptcmp#4-Column" = @{
        RegionName = "Footer Links"
        Metadata = [ordered]@{
            numberOfColumns = 4}
    }
    "$siteTypeCoreModuleLocationUrl/Editor/Templates/Include%20Page%20%28Footer%29.tptcmp#Link" = @{
        RegionName = "Links" }
    "$siteTypeCoreModuleLocationUrl/Editor/Templates/Include%20Page%20%28Footer%29.tptcmp#4-Column" = @{
        RegionName = "Footer Links"
        Metadata = [ordered]@{
            numberOfColumns = 4}
    }
    "$masterCoreModuleLocationUrl/Editor/Templates/Include%20Page%20%28Header%29.tptcmp#Logo" = @{
        RegionName = "Logo" }
    "$masterCoreModuleLocationUrl/Editor/Templates/Include%20Page%20%28Header%29.tptcmp#Nav" = @{
        RegionName = "Nav" }
    "$masterCoreModuleLocationUrl/Editor/Templates/Include%20Page%20%28Header%29.tptcmp#Info" = @{
    RegionName = "Info" }
    "$exampleSiteLocationUrl/Building%20Blocks/Modules/Core/Editor/Templates/Include%20Page%20%28Header%29.tptcmp#Logo" = @{
        RegionName = "Logo" }
    "$exampleSiteLocationUrl/Building%20Blocks/Modules/Core/Editor/Templates/Include%20Page%20%28Header%29.tptcmp#Nav" = @{
        RegionName = "Nav" }
    "$exampleSiteLocationUrl/Building%20Blocks/Modules/Core/Editor/Templates/Include%20Page%20%28Header%29.tptcmp#Info" = @{
        RegionName = "Info" }
    "$masterCoreModuleLocationUrl/Editor/Templates/Include%20Page%20%28Left%29.tptcmp#Left" = @{
        RegionName = "Left" }
    "$masterCoreModuleLocationUrl/Editor/Templates/Include%20Page%20%28Tools%29.tptcmp#Tools" = @{
        RegionName = "Tools" }
    "$masterCoreModuleLocationUrl/Editor/Templates/News%20Page.tptcmp#Main" = @{
        RegionName = "Article" }
    "$masterCoreModuleLocationUrl/Editor/Templates/News%20Page.tptcmp#" = @{
        RegionName = "Article" }
    "$masterCoreModuleLocationUrl/Editor/Templates/Section%20Page.tptcmp#Main" = @{
        RegionName = "Main" }
    "$masterCoreModuleLocationUrl/Editor/Templates/Section%20Page.tptcmp#" = @{
        RegionName = "Main" }
    "$masterCoreModuleLocationUrl/Editor/Templates/Section%20Page.tptcmp#Hero" = @{
        RegionName = "Hero" }
    "$masterCoreModuleLocationUrl/Editor/Templates/Section%20Page.tptcmp#3-Column" = @{
        RegionName = "Further Info"
        Metadata = [ordered]@{
            numberOfColumns = 3}
    }
    "$masterCoreModuleLocationUrl/Editor/Templates/Section%20Page.tptcmp#2-Column" = @{
        RegionName = "Further Info"
        Metadata = [ordered]@{
            numberOfColumns = 2}
    }
    "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Include%20Page.tptcmp#Main" = @{
        RegionName = "Main" }
    "$masterCoreModuleLocationUrl/Site%20Manager/Templates/Include%20Page.tptcmp#" = @{
        RegionName = "Main" }
}
}

$ScriptDir = Split-Path -parent $MyInvocation.MyCommand.Path
Import-Module $ScriptDir\CoreServiceUtils.ps1 -Force
Import-Module $ScriptDir\TcmItemUtils.ps1 -Force
Import-Module $ScriptDir\MessageUtils.ps1 -Force

function Set-NewPageTemplate
{
    param (
        [Parameter(Mandatory=$true)]
        [string]$pageId,

        [Parameter(Mandatory=$true)]
        [Object]$client,

        [Parameter(Mandatory=$false)]
        [string]$pageTemplateUri
    )

    $specificPageTemplate =  Get-NewPageTemplate $client $ItemMapperStorage["PageToSpecificPageTemplateMapper"][$pageId]

    if ($null -ne $specificPageTemplate)
    {
        Set-PageTemplate $client $page $specificPageTemplate
        $pageTemplateUri = $specificPageTemplate.Id
    }
    else
    {
        $newPageTemplate = Get-NewPageTemplate $client $ItemMapperStorage["PageTemplateMapper"][$pageTemplateUri]
        if ($null -ne $newPageTemplate)
        {
            Set-PageTemplate $client $page $newPageTemplate
        }
    }
    return $pageTemplateUri
}

function Get-NewPageTemplate
{
    param (
        [Parameter(Mandatory=$true)]
        [Object]$coreServiceClient,

        [Parameter(Mandatory=$false)]
        [string]$newPageTemplateUri
    )

    if ([string]::IsNullOrWhiteSpace($newPageTemplateUri))
    {
        Write-WarningMessage "PageTemplateMapper does not have map to '$PageTemplateUri'. Changing of the Page Template is skipped."
        return $null
    }
    $newPageTemplate = $coreServiceClient.Read($newPageTemplateUri, $null)
    return $newPageTemplate
}

function Transform-MappingTable
{
    param (
        [Parameter(Mandatory=$true)]
        [Object]$coreServiceClient
    )

    function Get-TcmUriForItem([string]$baseUri, [Object]$coreServiceClient)
    {
        if ([string]::IsNullOrWhiteSpace($baseUri))
        {
            return $baseUri
        }
        return $coreServiceClient.GetTcmUri($baseUri, $null, $null)
    }

    function Get-TransformedMapTableWithDefaultStructure([hashtable]$mapTable)
    {
        $transformedMap = @{}
        foreach ($item in $mapTable.GetEnumerator())
        {
            $newKey = Get-TcmUriForItem $item.Key $coreServiceClient
            $newValue = Get-TcmUriForItem $item.Value $coreServiceClient
            $transformedMap.Add($newKey, $newValue)
        }
        return $transformedMap
    }

    function Get-TransformedMapTableComponentPresentationMigration([hashtable]$mapTable)
    {
        $transformedMap = @{}
        foreach ($item in $mapTable.GetEnumerator())
        {
            $key = $item.Key
            $destinationRegion = "#"
            $separatorIndex = $key.LastIndexOf("#")
            if ($separatorIndex -ne -1)
            {
                $destinationRegion = $key.Substring($separatorIndex, $key.Length - $separatorIndex)
                $key = $key.Substring(0, $separatorIndex)
            }
            $pageTemplateId = Get-TcmUriForItem $key $coreServiceClient
            $newKey = $pageTemplateId + $destinationRegion
            $newValue = $item.Value
            $transformedMap.Add($newKey, $newValue)
        }
        return $transformedMap
    }

    Write-InfoMessage "Checking and transforming PageTemplateMapper... "
    $mapTable = Get-TransformedMapTableWithDefaultStructure $ItemMapperStorage["PageTemplateMapper"]
    $ItemMapperStorage["PageTemplateMapper"] = $mapTable
    Write-InfoMessage "Done"
    Write-InfoMessage "Checking and transforming ComponentTemplateMapper... "
    $mapTable = Get-TransformedMapTableWithDefaultStructure $ItemMapperStorage["ComponentTemplateMapper"]
    $ItemMapperStorage["ComponentTemplateMapper"] = $mapTable
    Write-InfoMessage "Done"
    Write-InfoMessage "Checking and transforming PageToSpecificPageTemplateMapper... "
    $mapTable = Get-TransformedMapTableWithDefaultStructure $ItemMapperStorage["PageToSpecificPageTemplateMapper"]
    $ItemMapperStorage["PageToSpecificPageTemplateMapper"] = $mapTable
    Write-InfoMessage "Done"
    $mapTable = Get-TransformedMapTableComponentPresentationMigration $ItemMapperStorage["ComponentPresentationDestinationRegionMap"]
    $ItemMapperStorage["ComponentPresentationDestinationRegionMap"] = $mapTable
    Write-InfoMessage "Done"
}

function Process-Page
{
    param (
        [Parameter(Mandatory=$true)]
        [string]$pageId,

        [Parameter(Mandatory=$false)]
        [string]$pageTemplateId,

        [Parameter(Mandatory=$true)]
        [Object]$client
    )

    $page = $client.Read($pageId, $null)
    $basePageSnapshot = Get-PageSnapshot $page
    Write-InfoMessage "Title : '$($page.Title)'"
    $newPageTemplate = Get-NewPageTemplate $client $pageTemplateId
    $pageTemplateUri = $newPageTemplate.Id
    Set-PageTemplate $client $page $newPageTemplate
    Set-PageRegions $page $client
    $componentPresentationsToRegionNamesMap = Get-ComponentPresentationsToRegionNamesMap $client $page.ComponentPresentations
    foreach ($componentPresentationToRegionNameItem in $componentPresentationsToRegionNamesMap.GetEnumerator()) {
        $componentPresentation = $componentPresentationToRegionNameItem.Key
        $regionName = $componentPresentationToRegionNameItem.Value
        Move-PageComponentPresentations $page $componentPresentation $pageTemplateUri $regionName $client
    }

    Set-NewComponentTemplate $page $ItemMapperStorage["ComponentTemplateMapper"]
    $newPageSnapshot = Get-PageSnapshot $page
    $isPageChanged = Get-SnapshotEquals $basePageSnapshot $newPageSnapshot
    if ($isPageChanged)
    {
        Write-InfoMessage "Page was not changed. Update skipped."
    }
    else
    {
        $client.Update($page, $null)
        Write-InfoMessage "Page was updated."
    }
    return $isPageChanged
}

function Main
{
    function Get-PageSnapshot($page)
    {
        return ConvertTo-Json -InputObject $page -Depth 5
    }

    function Get-SnapshotEquals($snapshotOne, $snapshotTwo)
    {
        return ($snapshotOne -eq $snapshotTwo)
    }

    $client = Get-CoreServiceClient $CmsUrl $AuthType $userName $userPass
    Transform-MappingTable $client
    $totalUpdatedPageCount = 0
    $totalProcessedPageCount = 0
    $updatedPageCount = 0
    $processedPageCount = 0
    foreach ($item in $ItemMapperStorage["PageToSpecificPageTemplateMapper"].GetEnumerator())
    {
        $pageId = $item.Key
        $pageTemplateId = $item.Value
        ++$processedPageCount
        Write-InfoMessage "Processing page ($processedPageCount/$allPageCount): '$($pageId)'"
        $isPageChanged = Process-Page $pageId $pageTemplateId $client
        if ($isPageChanged)
        {
            $updatedPageCount++
        }
    }
    $totalUpdatedPageCount += $updatedPageCount
    $totalProcessedPageCount += $processedPageCount
    Write-FormatedMessage "============================================================" -1 Cyan
    Write-FormatedMessage "Result report (Page to specific Page Template):"              -1 Cyan
    Write-FormatedMessage "    Processed pages: $processedPageCount"                     -1 Cyan
    Write-FormatedMessage "      Changed pages: $updatedPageCount"                       -1 Cyan
    Write-FormatedMessage "============================================================" -1 Cyan


    foreach ($mapper in $ItemMapperStorage["PageTemplateMapper"].GetEnumerator())
    {
        $updatedPageCount = 0
        $processedPageCount = 0
        $pageTemplateUri = $mapper.Key
        $pages = Get-PagesByPageTemplate $client $pageTemplateUri
        $allPageCount = $pages.Length
        foreach ($pageItem in $pages)
        {
            if ($ItemMapperStorage["PageToSpecificPageTemplateMapper"].ContainsKey($pageItem.Id))
            {
                continue
            }
            ++$processedPageCount
            Write-InfoMessage "Processing page ($processedPageCount/$allPageCount): '$($pageItem.Id)'"
            $isPageChanged = Process-Page $pageItem.Id $pageTemplateUri $client
            if ($isPageChanged)
            {
                $updatedPageCount++
            }
        }
        $totalUpdatedPageCount += $updatedPageCount
        $totalProcessedPageCount += $processedPageCount
        Write-FormatedMessage "============================================================" -1 Cyan
        Write-FormatedMessage "Result report (Pages with Page Template '$pageTemplateUri'):" -1 Cyan
        Write-FormatedMessage "    Processed pages: $processedPageCount"                     -1 Cyan
        Write-FormatedMessage "      Changed pages: $updatedPageCount"                       -1 Cyan
        Write-FormatedMessage "============================================================" -1 Cyan
    }
    $client.Close()
    Write-FormatedMessage "============================================================" -1 Cyan
    Write-FormatedMessage "Total result report"                                          -1 Cyan
    Write-FormatedMessage "    Processed pages: $totalProcessedPageCount"                -1 Cyan
    Write-FormatedMessage "      Changed pages: $totalUpdatedPageCount"                  -1 Cyan
    Write-FormatedMessage "============================================================" -1 Cyan
}

Main
